// Author : Pierre Vuillemin (2016)
// License : GNU GPL
mode(-1)
funcprot(0)

// =============================================================================
//                                  SOPI DEMO
//                          Linear optimisation problems
// =============================================================================
//

// Example 1
// -----------------------------------------------------------------------------
// Source: https://wiki.scilab.org/Linear%20Programming%20Examples%20in%20Scilab#Example_.231
// Minimize 2 * x1 + 9 * x2 + 3 * x3
// such that
//    2 * x1 - 2 * x2 - x3 <= -1
//    -x1 - 4 * x2 + x3 <= -1
//    x >= 0
// The solution is [0,1/3,1/3]
//
sopi_begin();           // sopi_begin must be called before any sopi-related operation
// Declaration of the optimisation variables
x1 = sopi_var(1, 'x1');
x2 = sopi_var(1, 'x2');
x3 = sopi_var(1, 'x3');
// Constraints
c1 = 2*x1 -2*x2-x3 <= -1;
c2 = -x1 - 4 * x2 + x3 <= -1;
c3 = x1 >= 0 & x2 >= 0 & x3 >= 0; // Constraints can be gathered with '&'
// Objective function
f  = 2*x1 + 9*x2 + 3*x3;
// Problem Instanciation
P  = sopi_min(f, c1, c2, c3);
// Problem resolution
sopi_solve(P) // By default, the built-in solver karmarkar is used
              // Other solvers can be used, linpro for example
// Optimal values are stored in the field 'opt'
mprintf('Example1:\n')
mprintf('Optimal value reached at for x=[%.2f,%.2f,%.2f]\n',x1.opt, x2.opt, x3.opt)
sopi_end() // Clean sopi-related variables and assign optimal values to x1, x2 and x3


// Example 2
// -----------------------------------------------------------------------------
// Source: https://wiki.scilab.org/Linear%20Programming%20Examples%20in%20Scilab#Example_.232
//Maximize 6*x1 + 4*x2 + 3*x3 + 2*x4
// such that
//    2*x1 + 3*x2 +   x3 + 2  *x4 <= 400
//      x1 +   x2 + 2*x3 +     x4 <= 150
//    2*x1 +   x2 +   x3 + 0.5*x4 <= 200
//    3*x1 +   x2 +            x4 <= 250
//                              x >= 0
// The solution is [50,100,0,0]
//
sopi_begin();
x1 = sopi_var(1, 'x1');
x2 = sopi_var(1, 'x2');
x3 = sopi_var(1, 'x3');
x4 = sopi_var(1, 'x4');
c1 = 2*x1 + 3*x2 +   x3 + 2  *x4 <= 400;
c2 = x1 + x2 + 2*x3 + x4 <= 150;
c3 = 2*x1 + x2 + x3 + 0.5*x4 <= 200;
c4 = 3*x1 + x2 + x4 <= 250;
b  = x1 >= 0 & x2 >= 0 & x3 >= 0 & x4 >= 0;
f  = 6*x1 + 4*x2 + 3*x3 + 2*x4; 
P  = sopi_max(f,c1,c2,c3,c4,b);
sopi_solve(P);
mprintf('Example2:\n')
mprintf('Optimal value for reached at x=[%.2f,%.2f,%.2f,%.2f]\n',x1.opt, x2.opt, x3.opt,x4.opt)
sopi_end()

// Example 3
// -----------------------------------------------------------------------------
// Source: https://wiki.scilab.org/Linear%20Programming%20Examples%20in%20Scilab#Example_.233
// Solution is [25, 20, 350]
sopi_begin();
x1 = sopi_var(1);
x2 = sopi_var(1);
x3 = sopi_var(1);
c  =           x1 + x2 <= 45  &...
     6*x1 + 10*x2 - x3 <= 0   & ...
                  5*x1 <= 140 & ...
                  4*x2 <= 120 & ...
                    x3 <= 350 & ...
                    x1 >= 0   & ...
                    x2 >= 0   & ...
                    x3 >= 0;
f  = 150*x1 + 200*x2 - 10*x3;
P = sopi_max(f,c);
sopi_solve(P)
mprintf('Example3:\n')
mprintf('Optimal value reached at for x=[%.2f,%.2f,%.2f]\n',x1.opt, x2.opt, x3.opt)
sopi_end()

// Example 4
// -----------------------------------------------------------------------------
// Source: https://wiki.scilab.org/Linear%20Programming%20Examples%20in%20Scilab#Example_.234
// Solution is [4,8]
sopi_begin()
x1 = sopi_var(1);
x2 = sopi_var(1);
f  = -20*x1 - 24*x2;
c  = 3*x1 + 6*x2 <= 60 & ...
     4*x1 + 2*x2 <= 32 & ...
              x1 >= 0  & ...
              x2 >= 0;
P = sopi_min(f,c);
sopi_solve(P)
mprintf('Example3:\n')
mprintf('Optimal value reached at for x=[%.2f,%.2f]\n',x1.opt, x2.opt)
// One can plot 1D or 2D problems directly
sopi_plotProblem(P,x1,x2);
sopi_end()
