// Author : Pierre Vuillemin (2016)
// License : GNU GPL
mode(-1)
funcprot(0)
// -----------------------------------------------------------------------------
//                                  SOPI DEMO
// -----------------------------------------------------------------------------
// This example shows how Sopi can be used to formulate and solve a simple nonlinear
// optimisation problem
//
// The considered problem is the following,
//       min_x        0.4 x^4 + 3*x^3 + 7*x + 10
//       s.t.
//             x >= -4
//
// -----------------------------------------------------------------------------

// sopi_begin must be called before any sopi-related operation
sopi_begin()
// Creation of the optimisation variables x and y
x        = sopi_var(1, 'x')
// Creation of the constraints of the problem. Note that constraints can be
// gathered with &
bound = x >= -4
// Creation of the objective function
objective = 0.4*x^4 + 3*x^3 + 7*x + 10
// Instanciation of the optimisation problem
problem           = sopi_min(objective, bound)
disp(sprintf('Optimisation problem built by sopi:\n'))
disp(problem)
// Resolution of the problem
[optVar, fopt, info] = sopi_solve(problem)
// The optimal values are given in the fields 'opt'
disp(sprintf('Solution of the problem:\n'))
disp(sprintf('\tSolver used: ''%s''', problem.solvers(problem.defaultSolver).name))
disp(sprintf('\tOptimal value of %s:%2.2f', x.name, x.opt))
disp(sprintf('\tOptimal objective value:%2.2f', objective.opt))
// Displaying the problem and the solution found
sopi_plotProblem(problem, x)
sopi_end()
