// Author : Pierre Vuillemin (2016)
// License : GNU GPL

function H = sopi_convexHull(x,y)
   // Computes the convex hull of a set of points in the plane
   //
   // Calling Sequence
   //    H = sopi_convexHull(x,y)
   //
   // Parameters
   //    x : x-axis coordinates of the points (array)
   //    y : y-axis coordinates (array)
   //    H : convex hull (array)
   //
   P  = [matrix(x,length(x),1),matrix(y,length(y),1)]
   Ps = sopi_sortRows(P,2)
   Ps = sopi_sortRows(P,1)
   //
   duplicatePoints = []
   for i = 2:size(Ps,1)
      if and(Ps(i,:) == Ps(i-1,:))
         duplicatePoints = [duplicatePoints, i]
      end
   end
   Ps(duplicatePoints,:) = []
   //
   N  = size(Ps,1)
   Lu = Ps(1:2,:)
   for i = 3:N
      Lu = [Lu;Ps(i,:)]
      while size(Lu,1) >2
         p1 = [Lu($-2,:),0]
         p2 = [Lu($-1,:),0]
         p3 = [Lu($,:),0]
         c = cross(p3-p1,p2-p1)
         if c(3) <=0
            Lu($-1,:) = []
         else
            break
         end
      end
   end
   Ll = [Ps($,:);Ps($-1,:)]
   for i = 3:N
      Ll = [Ll;Ps($-i+1,:)]
      while size(Ll,1)>2
         p3 = [Ll($,:),0]
         p2 = [Ll($-1,:),0]
         p1 = [Ll($-2,:),0]
         c = cross(p3-p1,p2-p1)
         if c(3)<0
            Ll($-1,:) = []
         else
            break
            end
         end
   end
   Ll(1,:) = []
   Ll($,:) = []
   H       = [Lu;Ll]
endfunction
