// Author : Pierre Vuillemin (2016)
// License : GNU GPL
function out = sopi_listSolvers()
   out = list()
   // ==========================================================================
   // LP SOLVERS
   // ==========================================================================
   //
   // karmarkar
   // --------------------------------------------------------------------------
   out($+1) = sopi_newSolver('name', 'karmarkar', 'objClass', 1,'cstClass', [0,1],'possibleOptions',['gam','maxiter','outfun','rtolf'])
   //
   // linpro
   // --------------------------------------------------------------------------
   if atomsIsLoaded('quapro')
      out($+1) = sopi_newSolver('name', 'linpro', 'objClass', 1, 'cstClass', [0,1])
   end
   // ==========================================================================
   // QP SOLVERS
   // ==========================================================================
   //
   // internal qp solver
   // --------------------------------------------------------------------------
   out($+1) = sopi_newSolver('name', 'int_qp', 'objClass', 20, 'cstClass', [-1])
   //
   // qld
   // --------------------------------------------------------------------------
   out($+1) = sopi_newSolver('name', 'qld', 'objClass', 20, 'cstClass', [0,1],'possibleOptions',['tol'])
   //
   // qpsolve
   // --------------------------------------------------------------------------
   out($+1) = sopi_newSolver('name', 'qpsolve', 'objClass', 20, 'cstClass', [0,1])   
   //
   // quapro
   // --------------------------------------------------------------------------
   if atomsIsLoaded('quapro')
      out($+1) = sopi_newSolver('name', 'quapro', 'objClass', [20,21,22,23,24], 'cstClass', [0,1])
   end
   // ==========================================================================
   // NL SOLVERS
   // ==========================================================================
   //
   // optim
   // --------------------------------------------------------------------------
   out($+1) = sopi_newSolver('name','optim','objClass',[1,42], 'cstClass',[-1,0],'requireX0', %T,'requireObjGrad',%T)
   //
   // fmincon
   // --------------------------------------------------------------------------
   if atomsIsLoaded('fmincon') then
      out($+1) = sopi_newSolver('name','fmincon', 'objClass',[1], 'cstClass', [0,1,2], 'requireX0', %T)
   end
   // ==========================================================================
   // STOCHASTIC
   // ==========================================================================
   //
   // cmaes
   // --------------------------------------------------------------------------
   if atomsIsLoaded('CMA-ES') then
      out($+1) = sopi_newSolver('name', 'cmaes','objClass', [1], 'cstClass', -1, 'requireX0', %T)
   end
endfunction


function solver = sopi_newSolver(varargin)
   objType  = 'sopiSolver';
   objAtt   = list( 'name'       , '',...
                    'objClass'   , [],...
                    'cstClass'   , [],...
                    'call'       , [],...
                    'acceptX0'   , %T,...
                    'requireX0'  , %F,...
                    'requireObjGrad', %F,...
                    'possibleOptions'  , []);
   solver = sopi_makeObject(objType, objAtt);
   solver = sopi_fillList(solver, varargin)
   if ~isempty(solver.name) then
      solver.call = 'sopi_' + solver.name
   end
endfunction
