// Author : Pierre Vuillemin (2017)
// License : GNU GPL


function [xopt, fopt, info] = sopi_qpsolve(problem, x0, options, optFields)
   // Attempt to solve a quadratic problem with the builtin method qpsole.
   //
   // Calling Sequence
   //    [xopt, fopt, info] = sopi_qpsolve(problem, x0, options, optFields)
   //
   // Parameters
   //    problem  : the optimisation problem to solve (sopiPro)
   //    x0       : the initial point (array, optional)
   //    options  : possible options for the solver (structure)
   //    optFields: name of the possible options (list)
   //    xopt     : optimal solution (array)
   //    fopt     : optimal objective function (double)
   //    info     : informations about the optimisation process (structure)
   //
   sopi_tryToExtract(options, optFields)
   // Constraints matrices
   C                    = [problem.Aeq;problem.A]
   b                    = [problem.beq;problem.b]
   //
  [xopt, iact,iter]   = qld(2*problem.fun.H,problem.fun.c,C,b,problem.lb, problem.ub,size(problem.Aeq,1))
   // Problem solution
   fopt = xopt'*problem.fun.H*xopt + problem.fun.c'*xopt + problem.fun.q
   // Information about the optimisation process
   info.solverInfo = struct('iter',iter,'iact',iact)
   info.vflag      = []
   info.success    = []
endfunction
