// Author : Pierre Vuillemin (2016)
// License : GNU GPL

function problem = sopi_addVar(problem, newVars)
   // Adds variables to an optimisation problem
   //
   // Calling Sequence
   //    problem = sopi_addVar(problem, newVars)
   //
   // Parameters
   //    problem: optimisation problem (sopiPro)
   //    newVars: new variables to add to the problem (list)
   //
   // Description
   //    This routine enables to add optimisation variables to an optimisation
   //    problem. The latter is updated with the new variables and the existing
   //    constraints are reshaped to match the new dimension of the decision
   //    vector.
   //
   varList = problem.varList
   idxList = problem.varIdx
   for var = newVars
      if isempty(sopi_findVarPosInList(var, varList)) then
         varList($+1)   = var
         nvar           = var.nvar
         problem        = sopi_increaseNvar(problem, nvar)
         if length(idxList) > 0 then
            offset = idxList($)($)
         else
            offset = 0
         end
         idxList($+1) = offset + 1 : offset + nvar
      end
   end
   problem.varList = varList
   problem.varIdx  = idxList
endfunction
