// Author : Pierre Vuillemin (2016)
// License : GNU GPL

function out = sopi_getVarIdxInProblem(problem, var)
   // Returns the indexes of a variable in an optimisation problem.
   //
   // Calling Sequence
   //    out = sopi_getVarIdxInProblem(problem, var)
   //
   // Parameters
   //    problem  : optimisation problem (sopiPro)
   //    var      : optimisation variable (sopiVar)
   //    out      : the indexes of the elements of var in the vector of the
   //               problem (array)
   //
   // Description
   //    When an optimisation problem is created, the variables are vectorised and
   //    stacked in one unique vector containing all the optimisation variables.
   //    This function returns the place of a variable in this large vector.
   position = sopi_findVarPosInList(var, problem.varList)
   if isempty(position) then
      out = []
   else
      out = problem.varIdx(position)
   end
endfunction
