// Author : Pierre Vuillemin (2016)
// License : GNU GPL

function [out, found] = sopi_extractPolyPart(var, order)
   // Extracts the coefficients and variables involved in a polynomial way.
   //
   // Calling Sequence
   //    out = sopi_extractPolyPart(var, order)
   //
   // Parameters
   //    var   : the variable where to look (sopiVar)
   //    order : the order of the polynomial part to extract (int)
   //    out   : a list containing the coefficients and variables (list)
   //
   // Description
   //
   // TODO : comment!
   found = %F
   out   = list()
   select var.operator
   case []
      //
      // Elementary variable
      //
      if order == 1
         // if the variable is elementary then its left and right linear parts are
         // identity matrices, i.e. z = I_1 * z * I_2
         [m, n]   = size(var)
         L        = eye(m,m)
         R        = eye(n,n)
         out      = list(list(list(L,R),list(var)))
         found    = %T
      end
   case 'transpose'
      //
      // Transposed variable
      //
      [inner, found] = sopi_extractPolyPart(var.args(1), order)
      if found then
         for i = 1:length(inner)
            nmats    = length(inner(i)(1))
            nvars    = length(inner(i)(2))
            out($+1) = list(list(),list())
            for j = 1:nvars
               out(i)(1)($+1) = inner(i)(1)(nmats - j + 1 )'
               out(i)(2)($+1) = inner(i)(2)(nvars - j + 1)'
            end
            out(i)(1)($+1) = inner(i)(1)(1)'
         end
      end
   case 'constant'
      //
      // Constant variable
      //
      if order == 0
         out   = list(list(list(var.args(1)),list()))
         found = %T
      end
   case 'sum'
      //
      // Sum of variables
      //
      for a = var.args
         [inner,foundi] = sopi_extractPolyPart(a, order)
         if foundi
            found = %T
            out   = lstcat(out, inner)
         end
      end
   case 'times'
      //
      // Product of variables
      //
      var1  = var.args(1)
      var2  = var.args(2)
      for i = 0:order
         [left, found1]    = sopi_extractPolyPart(var1, order - i)
         [right, found2]   = sopi_extractPolyPart(var2, i)
         if found1 & found2
            found = %T
            for j = 1:length(left)
               lj = left(j)
               for k = 1:length(right)
                  rk             = right(k)
                  lleft          = lj(1)($)
                  fright         = rk(1)(1)
                  rk(1)(1)       = null()
                  tmpLeft        = lj
                  tmpLeft(1)($)  = null()
                  fac            = lstcat(tmpLeft(1),list(lleft*fright))
                  fac            = lstcat(fac, rk(1))
                  vars           = lstcat(tmpLeft(2), rk(2))
                  out($+1)       = list(fac, vars)
               end
            end
         end
      end
   case "pow"
      //
      // Power of variable
      //
      [out, found] = sopi_extractPolyPart(var.args(1), order -var.args(2).args(1)+1)
      // TODO : ajouter les facteurs "centraux" dans des expressions de type L * x^2 * R
      for i = 1:length(out)
         if length(out(i)(1)) == 2
            L              = out(i)(1)(1)
            R              = out(i)(1)(2)
            var            = out(i)(2)(1)
            out(i)(1)      = list()
            out(i)(2)      = list()
            out(i)(1)(1)   = L
            for j = 1:order-1
               out(i)(1)($+1) = 1
               out(i)(2)($+1) = var
            end
            out(i)(2)($+1) = var
            out(i)(1)($+1) = R
         end
      end
   end
   // Post treatment of the solution
   if order == 0 & found
      // all constant are added
      tmp = out
      out = 0
      for o = tmp
         out = out + o(1)(1)
      end
      out = list(list(list(out), list()))
   end
   // If nothing has been found, the variable is forced to empty
   if ~found
      out = []
   end
endfunction
