// Copyright 2016 - Pierre Vuillemin
// License : GNU GPL

function sopi_example(key)
   // Interface for loading examples for sopi.
   //
   // Calling Sequence
   //    sopi_example()
   //    sopi_example(key)
   //
   // Parameters
   //    key : code of the example to load
   //
   // Description
   //    With no argument, it displays the list of examples and their
   //    corresponding codes.
   //    With an argument, it opens the corresponding example.
   //
   // Examples
   //    sopi_example()       // list the available examples
   //    sopi_example('lp')   // load the file about linear problems
   //
   if argn(2) < 1 then
      // displaying the list of examples
      titl     = 'List of examples'
      headers  = list('Key','Description')
      entries  = list('''lp''', 'Linear optimisation problems')
      tab      = sopi_tabular(titl, headers, entries)
      mprintf(tab)
      mprintf('NB: type ''sopi_example(<example number>)'' \n    to load the corresponding example.\n')
   else
      // loading one example
      if atomsIsLoaded('sopi')
         src_path = atomsGetLoadedPath('sopi')
      else
         [tmp,build_path]  = libraryinfo('sopiToolbox')
         src_path          = part(build_path, 1:length(build_path) - 14)
      end
      src_path = src_path + filesep() + 'demos' + filesep() + 'sopi_demo_' + convstr(key, 'u')
      src_path    = src_path + '.sce'
      if ~isfile(src_path)
         error(sprintf('Unknown example ''%s''',key))
      end    
      target_path = TMPDIR + filesep() + 'sopi_example.sce'
      copyfile(src_path, target_path)
      if (isdef('editor') | (funptr('editor') <> 0)) then
         editor(target_path);
      end      
   end
endfunction
